import { css } from "@linaria/core";
import { upperFirst } from "lodash";
import { defineComponent } from "vue";
import { string } from "vue-types";
import { useEditor } from "../../..";
import { DesignComp } from "../../../objects/DesignTemp/DesignComp";
import { Transfer } from "./Transfer";

export const View = defineComponent({
  props: {
    compId: string(),
  },
  emits: ["dblclick"],
  setup(props, { slots, emit }) {
    const { store, actions, helper } = useEditor();
    const comp =
      (props.compId && store.designData.compMap[props.compId]) ||
      new DesignComp();

    function createStyle(): any {
      const style: any = {};
      const {
        alignSelf,
        margin,
        zIndex,
        visible,
        size = [],
      } = comp.layout || {};
      const [w, h] = size;
      if (alignSelf) {
        style.alignSelf = alignSelf;
      }
      if (visible === false) {
        style.display = "none";
      }
      // if (offsetY) {
      //   style[`margin` + ((offsetY as number) > 0 ? "Top" : "Bottom")] =
      //     helper.designToNaturalSize(Math.abs(offsetY as number) * -1);
      // }
      if (zIndex) {
        style["zIndex"] = zIndex;
      }

      if (margin) {
        style.margin = margin;
      }

      if (comp.layout?.padding) {
        style.padding = comp.layout.padding;
      }

      if (w) style["width"] = helper.designToNaturalSize(w);
      if (h) style["height"] = helper.designToNaturalSize(h);
      return style;
    }

    function createContentStyle() {
      const style: any = {};
      const { background, layout } = comp;
      const [w, h] = (layout?.size as number[]) || [];
      if (background) {
        Object.entries(background).forEach(([key, value]) => {
          if (key === "image") {
            value = `url(${value})`;
          }
          style["background" + upperFirst(key)] = value;
        });
      }
      if (layout?.padding) {
        style.padding = layout.padding;
      }

      // if (layout?.offsetX) {
      //   style["marginLeft"] = helper.designToNaturalSize(
      //     layout.offsetX as number
      //   );
      // }

      return style;
    }

    return () => {
      const isSelected = store.currCompId === props.compId;

      return (
        <div
          class={[viewStyle, store.isEditMode && viewEditStyle]}
          style={createStyle()}
          onClick={(e) => {
            e.stopPropagation();
            if (!isSelected) {
              actions.pickComp(props.compId as string);
            }
          }}
          onDblclick={() => emit("dblclick")}
        >
          {slots.default?.()}
          {isSelected && <Transfer compId={props.compId} />}
        </div>
      );
    };
  },
});

const viewStyle = css`
  position: relative;
  font-size: 0;

  > :first-child {
    width: 100%;
    height: 100%;
  }
`;

const viewEditStyle = css`
  &:hover {
    outline: 2px dashed @inf-primary-color;
  }
`;
